%h1 Programming Documents with Haml

%p
  %a{:href=>"http://haml.info/"} Haml
  is an templating language for writing HTML. Over the past decade the
  capabilities of HTML+CSS have become good enough for nearly everything:
  letters,
  = succeed ',' do
    %a{:href=>"http://eradman.com/resume/"} my resume
  pages on this site, even
  = succeed '.' do
    %a{:href=>"http://raticalphotography.com/articles/darkroom-printing.html"} forms

%p.warning
  Haml 6 uses a completely incompatible library and command line.  To work
  around this set the gem install to version 5.2.2.

%p
  Haml combines a set of valuable features:

%ol
  %li A notation that mirrors HTML
  %li Blocks of text can be interpreted using custom filters
  %li Ad-hoc scripting

%h2 Mirroring HTML

%p
  Notwithstanding the name, Haml is not very abstract.  It is mostly a 1:1
  representation of the HTML it generates

:codeblock
  :::haml
  !!!
  %html
    %head
      %title A Letter
      %link{:href=>"main.css", :rel=>"stylesheet", :type=>"text/css"}/
    %body
      = Haml::Engine.new('home.haml').render
      %hr/
      %p.timestamp
        - require 'date'
        = "Last updated on " + Time.now().strftime("%B %d, %Y")

%p
  Setting a
  %code class
  and
  %code id properties
  have their own syntax

:codeblock
  :::haml
  / class="first"
  %p.first

  / id="content"
  %p#content

%p
  Ruby code can be run directly, and
  %code Haml::Engine
  can be used directly to process another template.

%h2 Building

%p
  In Haml, Ruby itself is also not abstracted, which gives us the ability to
  invent new mechanisms. One simple scheme I have used is to read extra
  arguments on the command line

:codeblock
  :::haml
  = Haml::Engine.new(File.read(ARGV[-1])).render

%p
  This means Makefile can easily communicate a source file to read

:codeblock
  :::make
  .SUFFIXES: .haml .html

  SOURCES != ls posts/*.haml | awk 'sub("\.haml$$", ".html")'

  haml.html:
  	haml template.haml $@ $<

%h2 Filters

%p
  Perhaps the most important feature of template language is the ability to
  process and transform blocks of text

:codeblock
  :::ruby
  # link.rb
  module Haml::Filters::Link
    include Haml::Filters::Base

    def render(text)
      %q|<a href="\#{text}">\#{text}</a>|
    end
  end

%p
  This function processes a block of text as the input

:codeblock
  :::haml
  %p
    Visit my home page at
    :link
      http://eradman.com/

%p
  And to compile add an include path and a reference to the library

:codeblock
  :::sh
  haml -I . -r link my.haml

%h2 Built-in Tests

%p
  Builtin self tests are an excellent means of writing filters.  Haml filters
  are simple Ruby modules, so we can build simple self-tests without having to
  instantiate an object

:codeblock
  :::ruby
  require 'haml'

  module Haml::Filters::Link
      def render(text)
          # ...
      end
  end

  if __FILE__ == $0
      include Haml::Filters::Link

      out = render("http://eradman.com")
      expected = %q|<a href="http://eradman.com">http://eradman.com</a>|

      if expected == out
          puts "PASS"
      else
          puts <<~MESSAGE
            FAIL
            expected: \#{expected}
            got     : \#{out}
          MESSAGE
      end
  end

%h2 Whitespace

%p
  Nearly every other templating system seems to have difficulty controlling
  whitespace, and Haml is no exception.

%p
  The normal way to add a comma or a period after another tag is to use the
  and
  %code succeed
  method. (See also
  = succeed ').' do
    %code surround
  In this example I want a period immediately following the hyperlink

:codeblock
  :::haml
  Send comments to
  = succeed '.' do
    %a{:href=>"mailto:ericshane@eradman.com"} ericshane@eradman.com

%p
  Another technique for controlling whitespace is to append
  %code &gt;
  to trim whitespace and then add non-breaking spaces where needed

:codeblock
  :::haml
  %pre
    %span.Prompt> $&nbsp;
    ls *.css *.html | entr reload-browser Firefox

%p
  Note that Haml allows HTML to be mixed in directly.  This is a wonderful
  option whenever raw HTML provides a readable or correct solution to
  formatting.
